/*

Intro to Query Tuning
January 2010
Mike DeFehr
www.mikedefehr.com
winnipeg.sqlpass.org

*/

SET STATISTICS IO ON
--Turn on show plan (Ctrl-M)

--to tune queries, we need more info about our tables
EXEC sp_help 'ProductOrder'
EXEC sp_helpindex 'ProductOrderDelivery'
EXEC sp_SQLskills_SQL2008_helpindex 'ProductOrderDelivery'		--Kimberly Tripp sp_helpindex8 SQL 2005/2008

--what does the plan for a simple query look like?
SELECT *
FROM ProductOrder
WHERE ProductOrderID = 12

--does this query have a good plan?
SELECT *
FROM ProductOrder
WHERE CustomerPOID = '12'

--join-through query
SELECT c.CustomerName 
FROM ProductOrderDelivery pd
	JOIN ProductOrderLine pl ON pd.ProductOrderLineID = pl.ProductOrderLineID
	JOIN ProductOrder po ON pl.ProductOrderID = po.ProductOrderID
	JOIN Customer c ON po.CustomerID = c.CustomerID
WHERE pd.ProductOrderDeliveryID = 18
OPTION (FORCE ORDER)

--how does sql server process?  is this the same query?
SELECT c.CustomerName
FROM Customer c
	JOIN ProductOrder po ON c.CustomerID = po.CustomerID
	JOIN ProductOrderLine pl ON po.ProductOrderID = pl.ProductOrderID
	JOIN ProductOrderDelivery pd ON pl.ProductOrderLineID = pd.ProductOrderLineID
WHERE pd.ProductOrderDeliveryID = 18
OPTION (FORCE ORDER)

--another join-through query
SELECT TOP 1 pd.ProductOrderDeliveryID
FROM Customer c
	JOIN ProductOrder po ON c.CustomerID = po.CustomerID
	JOIN ProductOrderLine pl ON po.ProductOrderID = pl.ProductOrderID
	JOIN ProductOrderDelivery pd ON pl.ProductOrderLineID = pd.ProductOrderLineID
WHERE c.CustomerID = 1
OPTION (FORCE ORDER, MAXDOP 1)

EXEC sp_SQLskills_SQL2008_helpindex 'ProductOrderDelivery'

EXEC sp_SQLskills_SQL2008_helpindex 'Customer';

--create indexes on all foreign keys in a database (note: assumes simple, one-column keys)
WITH cte AS
(
	SELECT	pst.name AS ParentTable, 
			psc.name AS ParentColumn
	FROM sys.foreign_keys fk
		JOIN sys.tables pst ON fk.parent_object_id = pst.[object_id]
		JOIN sys.foreign_key_columns fkc ON fk.[object_id] = fkc.constraint_object_id
		JOIN sys.columns psc ON fkc.parent_column_id = psc.column_id AND fk.parent_object_id = psc.[object_id]
)
--SELECT 'DROP INDEX ' + ParentTable + '.ix_' + ParentTable + '_' + ParentColumn
SELECT 'CREATE INDEX ix_' + ParentTable + '_' + ParentColumn + ' ON [' + ParentTable + '] (' + ParentColumn + ')'
FROM cte


CREATE INDEX ix_CustomerShipTo_CustomerID ON [CustomerShipTo] (CustomerID)
CREATE INDEX ix_Product_ProductCategoryID ON [Product] (ProductCategoryID)
CREATE INDEX ix_Product_CustomerID ON [Product] (CustomerID)
CREATE INDEX ix_ProductOrder_CustomerID ON [ProductOrder] (CustomerID)
CREATE INDEX ix_ProductOrderLine_ProductOrderID ON [ProductOrderLine] (ProductOrderID)
CREATE INDEX ix_ProductOrderLine_ProductID ON [ProductOrderLine] (ProductID)
CREATE INDEX ix_ProductOrderDelivery_ProductOrderLineID ON [ProductOrderDelivery] (ProductOrderLineID)
CREATE INDEX ix_ProductOrderDelivery_CustomerShipToID ON [ProductOrderDelivery] (CustomerShipToID)


--The tipping point

SELECT	CustomerID, 
		COUNT(*) AS NumOrders,
		COUNT(*) / CAST(COUNT(*) OVER() AS float) AS OrdPercent
FROM ProductOrder
GROUP BY CustomerID
ORDER BY 2 DESC
COMPUTE SUM(COUNT(*))

SELECT OrderDate
FROM ProductOrder
WHERE CustomerID = 22

CREATE INDEX ix_ProductOrder_CustomerID ON ProductOrder (CustomerID)
	INCLUDE(OrderDate) WITH DROP_EXISTING

--Looking at the tipping point within join-through queries

SELECT c.CustomerName, po.OrderDate, p.ProductName, pd.OrderQty, pd.ProductPrice, pd.DeliveryDate, cs.ShipToAddress
FROM Customer c
	JOIN ProductOrder po ON c.CustomerID = po.CustomerID
	JOIN ProductOrderLine pl ON po.ProductOrderID = pl.ProductOrderID
	JOIN Product p ON pl.ProductID = p.ProductID
	JOIN ProductOrderDelivery pd ON pl.ProductOrderLineID = pd.ProductOrderLineID 
	JOIN CustomerShipTo cs ON pd.CustomerShipToID = cs.CustomerShipToID
WHERE po.ProductOrderID = 501


SELECT c.CustomerName, po.OrderDate, p.ProductName, pd.OrderQty, pd.ProductPrice, pd.DeliveryDate, cs.ShipToAddress
FROM Customer c
	JOIN ProductOrder po ON c.CustomerID = po.CustomerID
	JOIN ProductOrderLine pl ON po.ProductOrderID = pl.ProductOrderID
	JOIN Product p ON pl.ProductID = p.ProductID
	JOIN ProductOrderDelivery pd ON pl.ProductOrderLineID = pd.ProductOrderLineID 
	JOIN CustomerShipTo cs ON pd.CustomerShipToID = cs.CustomerShipToID
WHERE po.ProductOrderID = 501
OPTION (FORCE ORDER)







--what happens if we include *almost* all of the columns?
CREATE INDEX ix_ProductOrderDelivery_ProductOrderLineID ON ProductOrderDelivery(ProductOrderLineID)
	INCLUDE (OrderQty, ProductPrice, DeliveryDate) WITH DROP_EXISTING
	
CREATE INDEX ix_ProductOrderDelivery_ProductOrderLineID ON ProductOrderDelivery(ProductOrderLineID)
	INCLUDE (OrderQty, ProductPrice, DeliveryDate,CustomerShipToID) WITH DROP_EXISTING
	
--one more thing to optimize
CREATE INDEX ix_ProductOrderLine_ProductOrderID ON ProductOrderLine(ProductOrderID)
	INCLUDE(ProductID) WITH DROP_EXISTING



