/*

Intro to Query Tuning - Set up the test database
January 2010
Mike DeFehr
www.mikedefehr.com
winnipeg.sqlpass.org

*/
SET NOCOUNT ON

USE master
GO
IF EXISTS(SELECT * FROM sys.databases WHERE name = 'TestQT')
BEGIN
	ALTER DATABASE TestQT SET RESTRICTED_USER WITH ROLLBACK IMMEDIATE
	DROP DATABASE TestQT
END
GO

DECLARE @Data varchar(1000), @Log varchar(1000), @cmd nvarchar(max)
exec master.dbo.xp_instance_regread N'hkey_local_machine',N'software\microsoft\mssqlserver\mssqlserver',N'defaultdata', @Data output
exec master.dbo.xp_instance_regread N'hkey_local_machine',N'software\microsoft\mssqlserver\mssqlserver',N'defaultlog', @Log output
SET @Data = @Data + '\TestQT.MDF'
SET @Log = @Log + '\TestQT_log.LDF'
--SELECT @Data, @Log

SET @cmd = 'CREATE DATABASE TestQT
	ON (NAME = ''TestQT'', FILENAME = ''' + @Data + ''', SIZE=300MB, FILEGROWTH=50MB)
	LOG ON (NAME=''TestQT_log'',FILENAME = ''' + @Log + ''',SIZE=50MB, FILEGROWTH=50MB)'
EXEC (@cmd)
GO
ALTER DATABASE TestQT SET RECOVERY SIMPLE
GO
USE TestQT
GO

CREATE TABLE Customer (
	CustomerID int IDENTITY(1,1) CONSTRAINT PK_Customer_CustomerID PRIMARY KEY,
	CustomerName varchar(255),
	CustomerInformation nchar(100) NOT NULL DEFAULT(CAST(NEWID() AS nchar(100)))
)
CREATE TABLE CustomerShipTo (
	CustomerShipToID int IDENTITY(1,1) CONSTRAINT PK_CustomerShipTo_CustomerShipToID PRIMARY KEY,
	CustomerID int
		CONSTRAINT FK_CustomerShipTo_Customer
		FOREIGN KEY REFERENCES Customer(CustomerID),
	ShipToAddress varchar(100),
	ShipToInformation nchar(1000) NOT NULL DEFAULT(CAST(NEWID() AS nchar(1000)))
)
CREATE TABLE ProductCategory (
	ProductCategoryID int IDENTITY(1,1) CONSTRAINT PK_ProductCategory_ProductCategoryID PRIMARY KEY,
	ProductCategoryName varchar(255),
	CategoryInformation nchar(1000) NOT NULL DEFAULT(CAST(NEWID() AS nchar(1000)))
)
CREATE TABLE Product (
	ProductID int IDENTITY(1,1) CONSTRAINT PK_Product_ProductID PRIMARY KEY,
	ProductName varchar(255),
	ProductPrice money,
	ProductCategoryID int 
		CONSTRAINT FK_Product_ProductCategory
		FOREIGN KEY REFERENCES ProductCategory(ProductCategoryID),
	CustomerID int 
		CONSTRAINT FK_Product_Customer
		FOREIGN KEY REFERENCES Customer(CustomerID),
	ProductInformation nchar(1000) NOT NULL DEFAULT(CAST(NEWID() AS nchar(1000)))
)
CREATE TABLE ProductOrder (
	ProductOrderID int IDENTITY(1,1) CONSTRAINT PK_ProductOrder_ProductOrderID PRIMARY KEY,
	CustomerID int
		CONSTRAINT FK_ProductOrder_Customer
		FOREIGN KEY REFERENCES Customer(CustomerID),
	CustomerPOID varchar(100),
	OrderDate date,
	OrderInformation char(36) NOT NULL DEFAULT(CAST(NEWID() AS char(36)))
)

CREATE TABLE ProductOrderLine (
	ProductOrderLineID int IDENTITY(1,1) CONSTRAINT PK_ProductOrderLine_ProductOrderLineID PRIMARY KEY,
	ProductOrderID int 
		CONSTRAINT FK_ProductOrderLine_ProductOrder
		FOREIGN KEY REFERENCES ProductOrder (ProductOrderID),
	ProductID int
		CONSTRAINT FK_ProductOrderLine_Product
		FOREIGN KEY REFERENCES Product(ProductID),
	OrderLineInformation nchar(200) NOT NULL DEFAULT(CAST(NEWID() AS nchar(200)))
)
CREATE TABLE ProductOrderDelivery (
	ProductOrderDeliveryID int IDENTITY(1,1) CONSTRAINT PK_ProductOrderDelivery_ProductOrderDeliveryID PRIMARY KEY,
	ProductOrderLineID int
		CONSTRAINT FK_ProductOrderDelivery_ProductOrderLine
		FOREIGN KEY REFERENCES ProductOrderLine(ProductOrderLineID),
	CustomerShipToID int
		CONSTRAINT FK_ProductOrderDelivery_CustomerShipTo
		FOREIGN KEY REFERENCES CustomerShipTo(CustomerShipToID),
	OrderQty int,
	ProductPrice money,
	DeliveryDate date,
	OrderDeliveryInformation nchar(200) NOT NULL DEFAULT(CAST(NEWID() AS nchar(200)))
)
CREATE TABLE nums (n int CONSTRAINT PK_nums_n PRIMARY KEY);
GO
WITH cte AS (
	SELECT ROW_NUMBER() OVER(ORDER BY (SELECT 0)) AS n
	FROM sys.all_objects a CROSS JOIN sys.all_objects b
)
INSERT nums
SELECT n
FROM cte
WHERE n <= 50000

INSERT Customer (CustomerName)
SELECT 'Customer ' + CAST(n AS varchar)
FROM nums
WHERE n <= 50

INSERT CustomerShipTo (CustomerID,ShipToAddress)
SELECT CustomerID, ShipToAddress
FROM (SELECT NEWID() AS new_id, * FROM Customer) c
	CROSS APPLY (	SELECT 'Customer Ship to address #' + CAST(n as varchar) + ' for Customer ID ' + CAST(CustomerID AS varchar) AS ShipToAddress
					FROM nums WHERE n <= RAND(CHECKSUM(new_id)) * 10 + 2 ) a

INSERT ProductCategory (ProductCategoryName)
SELECT 'Category ' + CAST(n AS varchar)
FROM nums
WHERE n <= 10

INSERT Product (ProductName, ProductPrice, ProductCategoryID, CustomerID)
SELECT	'Product ' + CAST(n AS varchar),
		RAND(CHECKSUM(NEWID())) * 10 + 10,
		CAST(LOG(RAND(CHECKSUM(NEWID()))) * -1 AS int) + 1,
		CAST(RAND(CHECKSUM(NEWID())) * 50 AS int) + 1
FROM nums
WHERE n <= 1000;

--SELECT ProductCategoryID, COUNT(*)
--FROM Product
--GROUP BY ProductCategoryID
--ORDER BY 1

--SELECT CustomerID, COUNT(*)
--FROM Product
--GROUP BY CustomerID
--ORDER BY 1

WITH rnd AS
(
	SELECT RAND(CHECKSUM(NEWID()))AS r
	FROM nums WHERE n <= 20000
), rnd2 AS
(
	SELECT TOP 10000 r
	FROM rnd WHERE r > 0.00005
)
INSERT ProductOrder (CustomerID,OrderDate,OrderInformation)
SELECT	CAST(LOG(r)*-4.9 AS int) + 1,
		DATEADD(day,CAST(RAND(r) * 360 AS int),'2009-01-01'),
		CAST(NEWID() AS CHAR(36))
FROM rnd2

UPDATE ProductOrder SET CustomerPOID = ProductOrderID
--SELECT	CustomerID,	COUNT(*)
--FROM ProductOrder GROUP BY CustomerID ORDER BY 1

--INSERT ProductOrderLine (ProductID, ProductOrderID)
--SELECT ProductID, ProductOrderID
----SELECT COUNT(*)
--FROM Product p
--	JOIN ProductOrder o ON p.CustomerID = o.CustomerID


DECLARE @IterCount int = 0

RepeatProductOrderLineInsert:

SET @IterCount+=1
PRINT 'ProductOrderLineInsert ' + CAST(@IterCount as varchar);
WITH cte AS
(
SELECT *, ROW_NUMBER() OVER (PARTITION BY CustomerID ORDER BY ProductOrderID) AS OrderIndex
FROM ProductOrder
)
INSERT ProductOrderLine(ProductOrderID,ProductID) 
SELECT cte.ProductOrderID, p.ProductID 
FROM cte
	CROSS APPLY (	SELECT TOP 30 PERCENT *
					FROM (	SELECT NEWID() AS sort_by,*
							FROM Product
							WHERE CustomerID = cte.CustomerID) a
					ORDER BY sort_by ) p
WHERE OrderIndex = @IterCount
IF @@ROWCOUNT <> 0 GOTO RepeatProductOrderLineInsert

IF OBJECT_ID('tempdb..#ProductOrderDelivery') IS NOT NULL DROP TABLE #ProductOrderDelivery
CREATE TABLE #ProductOrderDelivery (
	ProductOrderDeliveryID int IDENTITY(1,1) CONSTRAINT PK_ProductOrderDelivery_ProductOrderDeliveryID PRIMARY KEY,
	ProductOrderLineID int,
	CustomerShipToID int,
	OrderQty int,
	ProductPrice money,
	DeliveryDate date
)

	
PRINT 'ProductOrderDelivery insert:'
INSERT #ProductOrderDelivery (ProductOrderLineID, CustomerShipToID, OrderQty, ProductPrice,DeliveryDate)
SELECT ProductOrderLineID, 
		CustomerShipToID,
		CAST(RAND(CHECKSUM(NEWID())) * 990 AS int) + 10,
		RAND(CHECKSUM(NEWID())) * 10 + 10,
		DATEADD(day,CAST(RAND(CHECKSUM(NEWID())) * 60 AS int),OrderDate)
FROM ProductOrderLine pl
	JOIN ProductOrder po ON pl.ProductOrderID = po.ProductOrderID
	JOIN CustomerShipTo cs ON po.CustomerID = cs.CustomerID


SET IDENTITY_INSERT ProductOrderDelivery ON
DECLARE @Count int = 0

-- do 1000 ProductOrderDelivery inserts at a time so that we don't end up with a huge log file that's a pain to get rid of

RepeatPODInsert:

SET @Count += 1

PRINT 'ProductOrderDelivery insert ' + CAST(@Count AS varchar)

INSERT ProductOrderDelivery (ProductOrderDeliveryID, ProductOrderLineID, CustomerShipToID, OrderQty, ProductPrice, DeliveryDate)
SELECT * FROM #ProductOrderDelivery WHERE ProductOrderDeliveryID BETWEEN (@Count-1)*1000+1 AND @Count * 1000
IF @@ROWCOUNT <> 0 GOTO RepeatPODInsert

SET IDENTITY_INSERT ProductOrderDelivery OFF


/*
--Take a backup of the database so that setting it up in the future is quicker


DECLARE @Backup varchar(1000)
exec master.dbo.xp_instance_regread N'hkey_local_machine',N'software\microsoft\mssqlserver\mssqlserver',N'backupdirectory', @Backup output
SET @Backup = @Backup + '\TestQT.BAK'

PRINT @Backup

BACKUP DATABASE TestQT TO DISK = @Backup WITH INIT, FORMAT
	,COMPRESSION --use compression if you have developer or enterprise editions
	

--Here's the restore command:


USE master
GO
IF EXISTS(SELECT * FROM sys.databases WHERE name = 'TestQT')
BEGIN	--this part will disconnect any sessions
	ALTER DATABASE TestQT SET RESTRICTED_USER WITH ROLLBACK IMMEDIATE
	ALTER DATABASE TestQT SET RECOVERY SIMPLE
	DROP DATABASE TestQT
END
DECLARE @Data varchar(1000), @Log varchar(1000), @Backup varchar(1000), @cmd nvarchar(max)
exec master.dbo.xp_instance_regread N'hkey_local_machine',N'software\microsoft\mssqlserver\mssqlserver',N'defaultdata', @Data output
exec master.dbo.xp_instance_regread N'hkey_local_machine',N'software\microsoft\mssqlserver\mssqlserver',N'defaultlog', @Log output
exec master.dbo.xp_instance_regread N'hkey_local_machine',N'software\microsoft\mssqlserver\mssqlserver',N'backupdirectory', @Backup output
SET @Data = @Data + '\TestQT.MDF'
SET @Log = @Log + '\TestQT_log.LDF'
SET @Backup = @Backup + '\TestQT.BAK'

RESTORE DATABASE TestQT FROM DISK = @Backup
	WITH MOVE 'TestQT' TO @Data,
	MOVE 'TestQT_log' TO @Log



*/